//Bibliotheken, die zu importieren sind
#include "DHT.h"  //Bibliothek des Luftfeuchtigkeits- und Temperatursensors
#include "Servo.h" // Bibliothek des Servomotors

#define DHT_PIN 5   //Daten Pin des DHT 
#define SERVO_PIN 6 //Daten Pin des Servos
#define LUM_PIN A0  //Analog Pin des Spannungsteilers für die Helligkeitsmessung
#define PROGRAM_HUM_PIN 3 // Pin für den Hubtaster des Programms zur Luftfeuchtigkeitsmessung
#define PROGRAM_LICHT_PIN 4 //Pin für den Hubtaster des Programms zur Lichtintensitätsmessung
#define HUM_DELTA 1 //Luftfeuchtigkeitsunterschied, ab dem der Servomotor angehen soll, 1 steht für eine Differnez von 1% der Luftfeuchtigkeit zum vorherigen messwert
#define DHT_TYPE DHT22  //Art des DHT Sensors
#define SPEAKER_PIN 7 // Pin für den Lautsprecher

#define LUM_MAX 1023 //Maximalwert für die Helligkeit
#define LUM_MIN 0   //Minimalwert für die Helligkeit

int servo_winkel; //Variable des aktuellen Servowinkels
float hum = -10;  // Variable für die aktuelle Luftfeuchtigkeit. Initialisiert bei -10, damit sich der Servo definitiv bewegt.

DHT my_dht(DHT_PIN, DHT_TYPE);  //Initialisierung des DHT Sensors
Servo my_Servo; //Initialisierung des Servomotors

void setup() {
  Serial.begin(9600); //Beginnt die Serielle Kommunikation. Notwendig für das Erhalten von Fehlermeldungen und die Kommunikation mit dem Arduino
  pinMode(PROGRAM_HUM_PIN,INPUT_PULLUP);  // Setzen des Pins für den Hubtaster der Luftfeuchtigkeitsmessung als Input Pin mit Pull Up Widerstand
  pinMode(PROGRAM_LICHT_PIN,INPUT_PULLUP);// Setzen des Pins für den Hubtaster der Helligkeitsmessung als Input Pin mit Pull Up Widerstand
  my_Servo.attach(SERVO_PIN); //Servomotor aktivieren. Der Servo beginnt zu summen und blockiert auf der Position.
  my_Servo.write(0);    //Servomotor zu Beginn auf 0 setzen
  
  my_dht.begin();   //DHT Sensor startet Messungen
  delay(1000);    //Pause von 1 Sekunde
  my_Servo.detach(); //Servomotor ausschalten. Der Motor kann jetzt wieder von hand bewegt werden. Grund für das Abschalten: Hoher Stromverbrauch des Servos und nerviges Summen
}

void loop() {
  //Überprüfung, ob einer der beiden Programmbuttons gedrückt wurde. Wenn nicht gedrückt haben die Butttons aufgrund des Pullup Widerstands jeweils den Wert 1.Sind sie gedrückt, ist der Wert 0
  if(digitalRead(PROGRAM_HUM_PIN) == 0){
    missLuftfeuchtigkeit(); //Messen der Luftfeuchtigkeit
    delay(1000);// 1 Sekunde warten nach der Messung
  }else if(digitalRead(PROGRAM_LICHT_PIN) == 0){
    findeLichtquelle(); //Lichtquelle erkennen
  }
  delay(10); 
}

//Funktion zum Finden der Lichtquellen. Am Analogpin wird der Wert des Spannungsteilers ausgelesen. Mit dem gemessenen Wert wird dann der Lautsprecher angesteuert. Umso heller das Licht, desto höher der Ton.
void findeLichtquelle(){
    int lum = analogRead(LUM_PIN);// Auslesen der Spannung am Spannungsteiler.
    int pitch = map(lum, LUM_MIN,LUM_MAX, 50, 1300); //Der Analogwert wird auf eine Skala zwischen LUM_MIN und LUM_MAX skaliert. Diese Skala wird dann auf einen Ton zwischen 50 und 1300 Hertz skaliert.
    tone(SPEAKER_PIN, pitch, 100); //Ton für 100 Millisekunden ausgeben.

}

//Messen der Luftfeuchtigkeit und Stellen des Servomotors in Abhängigkeit der Messung
void missLuftfeuchtigkeit(){
  //vergleicht, ob der Absolute Wert der Differenz zwischen gemessener Luftfeuchtigkeit und aktuell angezeigter Luftfeuchtigkeit größer ist, als die Obergrenze der Differenz zum Ändern der Servoposition, die festgelegt wurde.
  if (abs(my_dht.readHumidity() - hum) > HUM_DELTA) {
    my_Servo.attach(SERVO_PIN);    //falls ja, wird der Servomotor aktiviert. Das Summen beginnt und der Motor kann nicht mehr von Hand gedreht werden.
    hum = my_dht.readHumidity(); //Der Wert der Variablen hum wird mit dem aktuellen Wert der Luftfeuchtigkeit überschrieben
    servo_winkel = map(hum, 0, 100, 180, 0);  //Der Prozentwert der Luftfeuchtigkeit wird auf den Winkel des Servomotors gemappt. 0 bis 100% auf 0 bis 180 Grad
    my_Servo.write(servo_winkel); //Servowinkel stellt sich neu ein.
    delay(1000); // Pause
    my_Servo.detach();  //Servo deaktivieren
  }
}